@@ -18,9 +18,6 @@
 extern "C" {
 #endif
 
-//  Opaque class structure
-typedef struct _zmutex_t zmutex_t;
-
 //  @interface
 //  Create a new mutex container
 CZMQ_EXPORT zmutex_t *
@@ -42,9 +39,8 @@ CZMQ_EXPORT void
 CZMQ_EXPORT int
     zmutex_try_lock (zmutex_t *self);
 
-
 //  Self test of this class
-CZMQ_EXPORT int
+CZMQ_EXPORT void
     zmutex_test (bool verbose);
 //  @end
 
