@@ -1,4 +1,4 @@
-/*  =========================================================================
+﻿/*  =========================================================================
     zfile - provides methods to work with files in a portable fashion.
 
     Copyright (c) the Contributors as noted in the AUTHORS file.
@@ -30,7 +30,6 @@
 */
 
 #include "../include/czmq.h"
-#include "platform.h"
 
 //  Structure of our class
 
@@ -43,10 +42,12 @@ struct _zfile_t {
     bool eof;               //  true if at end of file
     FILE *handle;           //  Read/write handle
     zdigest_t *digest;      //  File digest, if known
+    char *curline;          //  Last read line, if any
+    size_t linemax;         //  Size of allocated buffer
 
     //  Properties from files that exist on file system
     time_t modified;        //  Modification time
-    off_t  cursize;         //  Size of the file
+    off_t cursize;          //  Size of the file
     mode_t mode;            //  POSIX permission bits
 };
 
@@ -64,35 +65,54 @@ zfile_new (const char *path, const char *name)
 {
     zfile_t *self = (zfile_t *) zmalloc (sizeof (zfile_t));
 
-    //  Format full path to file
-    if (path) {
-        self->fullname = (char *) zmalloc (strlen (path) + strlen (name) + 2);
-        sprintf (self->fullname, "%s/%s", path, name);
-    }
-    else
-        self->fullname = strdup (name);
-
-    //  Resolve symbolic link if possible
-    if (strlen (self->fullname) > 3
-    &&  streq (self->fullname + strlen (self->fullname) - 3, ".ln")) {
-        FILE *handle = fopen (self->fullname, "r");
-        if (handle) {
-            char buffer [256];
-            if (fgets (buffer, 256, handle)) {
-                //  We have the contents of the symbolic link
-                if (buffer [strlen (buffer) - 1] == '\n')
-                    buffer [strlen (buffer) - 1] = 0;
-                self->link = strdup (buffer);
-                //  Chop ".ln" off name for external use
-                self->fullname [strlen (self->fullname) - 3] = 0;
+    if (self) {
+        //  Format full path to file
+        if (path) {
+            self->fullname = (char *) zmalloc (strlen (path) + strlen (name) + 2);
+            if (self->fullname)
+                sprintf (self->fullname, "%s/%s", path, name);
+            else {
+                zfile_destroy (&self);
+                return NULL;
             }
-            fclose (handle);
         }
+        else
+            self->fullname = strdup (name);
+
+        if (self->fullname) {
+            //  Resolve symbolic link if possible
+            if (strlen (self->fullname) > 3
+            &&  streq (self->fullname + strlen (self->fullname) - 3, ".ln")) {
+                FILE *handle = fopen (self->fullname, "r");
+                if (handle) {
+                    char buffer [256];
+                    if (fgets (buffer, 256, handle)) {
+                        //  We have the contents of the symbolic link
+                        if (buffer [strlen (buffer) - 1] == '\n')
+                            buffer [strlen (buffer) - 1] = 0;
+                        self->link = strdup (buffer);
+                        
+                        //  Chop ".ln" off name for external use
+                        if (self->link)
+                            self->fullname [strlen (self->fullname) - 3] = 0;
+                        else {
+                            fclose (handle);
+                            zfile_destroy (&self);
+                            return NULL;
+                        }
+                    }
+                    fclose (handle);
+                }
+            }
+            zfile_restat (self);
+        }
+        else
+            zfile_destroy (&self);
     }
-    zfile_restat (self);
     return self;
 }
 
+
 //  --------------------------------------------------------------------------
 //  Destroy a file item
 
@@ -106,6 +126,7 @@ zfile_destroy (zfile_t **self_p)
         if (self->handle)
             fclose (self->handle);
         free (self->fullname);
+        free (self->curline);
         free (self->link);
         free (self);
         *self_p = NULL;
@@ -114,28 +135,35 @@ zfile_destroy (zfile_t **self_p)
 
 
 //  --------------------------------------------------------------------------
-//  Duplicate a file item
+//  Duplicate a file item, returns a newly constructed item. If the file
+//  is null, or memory was exhausted, returns null.
 
 zfile_t *
 zfile_dup (zfile_t *self)
 {
-    zfile_t
-        *copy;
-
-    copy = (zfile_t *) zmalloc (sizeof (zfile_t));
-    copy->fullname = strdup (self->fullname);
-    copy->modified = self->modified;
-    copy->cursize = self->cursize;
-    copy->link = self->link? strdup (self->link): NULL;
-    copy->mode = self->mode;
-    return copy;
+    if (self) {
+        zfile_t *copy = (zfile_t *) zmalloc (sizeof (zfile_t));
+        if (copy)
+            copy->fullname = strdup (self->fullname);
+        if (copy->fullname) {
+            copy->modified = self->modified;
+            copy->cursize = self->cursize;
+            copy->link = self->link? strdup (self->link): NULL;
+            copy->mode = self->mode;
+        }
+        else
+            zfile_destroy (&copy);
+        return copy;
+    }
+    else
+        return NULL;
 }
 
 
 //  --------------------------------------------------------------------------
 //  Return file name, remove path if provided
 
-char *
+const char *
 zfile_filename (zfile_t *self, const char *path)
 {
     assert (self);
@@ -174,6 +202,7 @@ zfile_restat (zfile_t *self)
         self->mode = 0;
         self->stable = false;
     }
+    zdigest_destroy (&self->digest);
 }
 
 
@@ -209,7 +238,7 @@ bool
 zfile_is_directory (zfile_t *self)
 {
     assert (self);
-    return (self->mode & S_IFDIR) != 0;
+    return S_ISDIR (self->mode);
 }
 
 
@@ -221,7 +250,7 @@ bool
 zfile_is_regular (zfile_t *self)
 {
     assert (self);
-    return (self->mode & S_IFREG) != 0;
+    return S_ISREG (self->mode);
 }
 
 
@@ -234,7 +263,7 @@ bool
 zfile_is_readable (zfile_t *self)
 {
     assert (self);
-    return (self->mode & S_IREAD) != 0;
+    return (self->mode & S_IRUSR) != 0;
 }
 
 
@@ -247,7 +276,7 @@ bool
 zfile_is_writeable (zfile_t *self)
 {
     assert (self);
-    return (self->mode & S_IWRITE) != 0;
+    return (self->mode & S_IWUSR) != 0;
 }
 
 
@@ -264,6 +293,27 @@ zfile_is_stable (zfile_t *self)
 }
 
 
+//  --------------------------------------------------------------------------
+//  Return true if the file was changed on disk since the zfile_t object
+//  was created, or the last zfile_restat() call made on it. Note that the
+//  file will not necessarily yet be stable.
+
+bool
+zfile_has_changed (zfile_t *self)
+{
+    assert (self);
+    struct stat stat_buf;
+    char *real_name = self->link? self->link: self->fullname;
+    if (stat (real_name, &stat_buf) == 0) {
+        //  It's not a foolproof heuristic but catches most cases
+        if (stat_buf.st_mtime != self->modified
+        ||  stat_buf.st_size != self->cursize)
+            return true;
+    }
+    return false;
+}
+
+
 //  --------------------------------------------------------------------------
 //  Remove the file
 
@@ -311,44 +361,49 @@ zfile_input (zfile_t *self)
 int
 zfile_output (zfile_t *self)
 {
+    int rc;
     assert (self);
-    
+
+    //  Create file path if it doesn't exist
+    char *file_path = strdup (self->fullname);
+    if (!file_path)
+        return -1;
+    char *last_slash = strrchr (file_path, '/');
+    if (last_slash)
+        *last_slash = 0;
+
     //  Wipe symbolic link if that's what the file was
     if (self->link) {
         free (self->link);
         self->link = NULL;
     }
-    //  Create file path if it doesn't exist
-    char *file_path = strdup (self->fullname);
-    char *last_slash = strrchr (file_path, '/');
-    if (last_slash)
-        *last_slash = 0;
-    if (zsys_dir_create (file_path))
-        return -1;
+    rc = zsys_dir_create (file_path);
     free (file_path);
+    if (rc != 0)
+        return -1;
 
     //  Create file if it doesn't exist
     if (self->handle)
         zfile_close (self);
-    self->handle = fopen (self->fullname, "r+b"); 
-    if (!self->handle) {
+    
+    self->handle = fopen (self->fullname, "r+b");
+    if (!self->handle)
         self->handle = fopen (self->fullname, "w+b");
-        if (!self->handle)
-            self->handle = fopen (self->fullname, "w+b");
-    }
+    
     return self->handle? 0: -1;
 }
 
 
 //  --------------------------------------------------------------------------
 //  Read chunk from file at specified position. If this was the last chunk,
-//  sets self->eof. Returns a null chunk in case of error.
+//  sets the eof property. Returns a null chunk in case of error.
 
 zchunk_t *
 zfile_read (zfile_t *self, size_t bytes, off_t offset)
 {
     assert (self);
     assert (self->handle);
+    
     //  Calculate real number of bytes to read
     if (offset > self->cursize)
         bytes = 0;
@@ -356,8 +411,7 @@ zfile_read (zfile_t *self, size_t bytes, off_t offset)
     if (bytes > (size_t) (self->cursize - offset))
         bytes = (size_t) (self->cursize - offset);
 
-    int rc = fseek (self->handle, (long) offset, SEEK_SET);
-    if (rc == -1)
+    if (fseek (self->handle, (long) offset, SEEK_SET) == -1)
         return NULL;
 
     self->eof = false;
@@ -368,6 +422,17 @@ zfile_read (zfile_t *self, size_t bytes, off_t offset)
 }
 
 
+//  --------------------------------------------------------------------------
+//  Returns the eof property.
+
+bool
+zfile_eof (zfile_t *self)
+{
+    assert (self);
+    return self->eof;
+}
+
+
 //  --------------------------------------------------------------------------
 //  Write chunk to file at specified position
 //  Return 0 if OK, else -1
@@ -384,6 +449,45 @@ zfile_write (zfile_t *self, zchunk_t *chunk, off_t offset)
 }
 
 
+//  --------------------------------------------------------------------------
+//  Read next line of text from file. Returns a pointer to the text line,
+//  or NULL if there was nothing more to read from the file.
+
+const char *
+zfile_readln (zfile_t *self)
+{
+    assert (self);
+    assert (self->handle);
+
+    //  Opportunistically allocate line buffer if needed; we'll grow this
+    //  if needed, so the initial linemax is not a big deal
+    if (!self->curline) {
+        self->linemax = 512; 
+        self->curline = (char *) malloc (self->linemax);
+    }
+    uint char_nbr = 0;
+    while (true) {
+        int cur_char = fgetc (self->handle);
+        if (cur_char == '\r')
+            continue;               //  Skip CR in MS-DOS format files
+        else
+        if (cur_char == EOF && char_nbr == 0)
+            return NULL;            //  Signal end of file
+        else
+        if (cur_char == '\n' || cur_char == EOF)
+            cur_char = 0;
+        if (char_nbr == self->linemax - 1) {
+            self->linemax *= 2;
+            self->curline = (char *) realloc (self->curline, self->linemax);
+        }
+        self->curline [char_nbr++] = cur_char;
+        if (cur_char == 0)
+            break;
+    }
+    return self->curline;
+}
+
+
 //  --------------------------------------------------------------------------
 //  Close file, if open
 
@@ -394,8 +498,9 @@ zfile_close (zfile_t *self)
     if (self->handle) {
         fclose (self->handle);
         zfile_restat (self);
+        self->handle = 0;
+        self->eof = false;
     }
-    self->handle = 0;
 }
 
 
@@ -411,12 +516,12 @@ zfile_handle (zfile_t *self)
 
 
 //  --------------------------------------------------------------------------
-//  Calculate SHA1 digest for file, using zdigest class. Caller should not
-//  modify digest.
+//  Calculate SHA1 digest for file, using zdigest class.
 
-char *
+const char *
 zfile_digest (zfile_t *self)
 {
+    assert (self);
     if (!self->digest) {
         if (zfile_input (self) == -1)
             return NULL;            //  Problem reading file
@@ -426,16 +531,25 @@ zfile_digest (zfile_t *self)
         off_t offset = 0;
 
         self->digest = zdigest_new ();
+        if (!self->digest)
+            return NULL;
         zchunk_t *chunk = zfile_read (self, blocksz, offset);
         while (zchunk_size (chunk)) {
             zdigest_update (self->digest,
-                zchunk_data (chunk), zchunk_size (chunk));
+                            zchunk_data (chunk), zchunk_size (chunk));
             zchunk_destroy (&chunk);
-            offset += blocksz;
+
+            //  off_t is defined as long (32 bit on Windows, 64 bit otherwise)
+            //  This guards against overflow in both contexts.
+            if (blocksz > LONG_MAX - offset)
+                return NULL;
+
+            offset += (off_t) blocksz;
             chunk = zfile_read (self, blocksz, offset);
         }
         zchunk_destroy (&chunk);
-        zfile_close (self);
+        fclose (self->handle);
+        self->handle = 0;
     }
     return zdigest_string (self->digest);
 }
@@ -446,31 +560,49 @@ zfile_digest (zfile_t *self)
 //  and lets us do more interesting things. These functions were
 //  essentially about portability, so now sit in zsys.
 
-bool zfile_exists (const char *filename) {
+bool
+zfile_exists (const char *filename)
+{
     return zsys_file_exists (filename);
 }
-ssize_t zfile_size (const char *filename) {
+ssize_t
+zfile_size (const char *filename)
+{
     return zsys_file_size (filename);
 }
-mode_t zfile_mode (const char *filename) {
+mode_t
+zfile_mode (const char *filename)
+{
     return zsys_file_mode (filename);
 }
-int zfile_delete (const char *filename) {
+int
+zfile_delete (const char *filename)
+{
     return zsys_file_delete (filename);
 }
-bool zfile_stable (const char *filename) {
+bool
+zfile_stable (const char *filename)
+{
     return zsys_file_stable (filename);
 }
-int zfile_mkdir (const char *pathname) {
+int
+zfile_mkdir (const char *pathname)
+{
     return zsys_dir_create (pathname);
 }
-int zfile_rmdir (const char *pathname) {
+int
+zfile_rmdir (const char *pathname)
+{
     return zsys_dir_delete (pathname);
 }
-void zfile_mode_private (void) {
+void
+zfile_mode_private (void)
+{
     zsys_file_mode_private ();
 }
-void zfile_mode_default (void) {
+void
+zfile_mode_default (void)
+{
     zsys_file_mode_default ();
 }
 
@@ -478,23 +610,27 @@ void zfile_mode_default (void) {
 //  --------------------------------------------------------------------------
 //  Self test of this class
 
-int
+void
 zfile_test (bool verbose)
 {
     printf (" * zfile: ");
 
     //  @selftest
     zfile_t *file = zfile_new (NULL, "bilbo");
+    assert (file);
     assert (streq (zfile_filename (file, "."), "bilbo"));
     assert (zfile_is_readable (file) == false);
     zfile_destroy (&file);
 
     //  Create a test file in some random subdirectory
     file = zfile_new ("./this/is/a/test", "bilbo");
+    assert (file);
     int rc = zfile_output (file);
     assert (rc == 0);
     zchunk_t *chunk = zchunk_new (NULL, 100);
+    assert (chunk);
     zchunk_fill (chunk, 0, 100);
+
     //  Write 100 bytes at position 1,000,000 in the file
     rc = zfile_write (file, chunk, 1000000);
     assert (rc == 0);
@@ -502,42 +638,64 @@ zfile_test (bool verbose)
     zfile_close (file);
     assert (zfile_is_readable (file));
     assert (zfile_cursize (file) == 1000100);
-    zfile_restat (file);
     assert (!zfile_is_stable (file));
+    assert (zfile_digest (file));
+
+    //  Now truncate file from outside
+    int handle = open ("./this/is/a/test/bilbo", O_WRONLY | O_TRUNC | O_BINARY, 0);
+    assert (handle >= 0);
+    rc = write (handle, "Hello, World\n", 13);
+    assert (rc == 13);
+    close (handle);
+    assert (zfile_has_changed (file));
     zclock_sleep (1001);
+    assert (zfile_has_changed (file));
+
     assert (!zfile_is_stable (file));
     zfile_restat (file);
     assert (zfile_is_stable (file));
-    assert (streq (zfile_digest (file), "F6CA2B0E6609C2B556F651F46A5A14C86153D0BF"));
-    
+    assert (streq (zfile_digest (file), "4AB299C8AD6ED14F31923DD94F8B5F5CB89DFB54"));
+
     //  Check we can read from file
     rc = zfile_input (file);
     assert (rc == 0);
     chunk = zfile_read (file, 1000100, 0);
     assert (chunk);
-    assert (zchunk_size (chunk) == 1000100);
+    assert (zchunk_size (chunk) == 13);
     zchunk_destroy (&chunk);
     zfile_close (file);
 
+    //  Check we can read lines from file
+    rc = zfile_input (file);
+    assert (rc == 0);
+    const char *line = zfile_readln (file);
+    assert (streq (line, "Hello, World"));
+    line = zfile_readln (file);
+    assert (line == NULL);
+    zfile_close (file);
+
     //  Try some fun with symbolic links
     zfile_t *link = zfile_new ("./this/is/a/test", "bilbo.ln");
+    assert (link);
     rc = zfile_output (link);
     assert (rc == 0);
     fprintf (zfile_handle (link), "./this/is/a/test/bilbo\n");
     zfile_destroy (&link);
 
     link = zfile_new ("./this/is/a/test", "bilbo.ln");
+    assert (link);
     rc = zfile_input (link);
     assert (rc == 0);
     chunk = zfile_read (link, 1000100, 0);
     assert (chunk);
-    assert (zchunk_size (chunk) == 1000100);
+    assert (zchunk_size (chunk) == 13);
     zchunk_destroy (&chunk);
     zfile_destroy (&link);
 
     //  Remove file and directory
     zdir_t *dir = zdir_new ("./this", NULL);
-    assert (zdir_cursize (dir) == 2000200);
+    assert (dir);
+    assert (zdir_cursize (dir) == 26);
     zdir_remove (dir, true);
     assert (zdir_cursize (dir) == 0);
     zdir_destroy (&dir);
@@ -552,5 +710,4 @@ zfile_test (bool verbose)
     //  @end
 
     printf ("OK\n");
-    return 0;
 }
